// $Id: EventHandler.cc,v 1.3 2003/11/19 07:26:01 vern Exp $

#include "EventHandler.h"
#include "Func.h"
#include "Scope.h"
#include "RemoteSerializer.h"

EventHandler::EventHandler(const char* arg_name)
	{
	name = copy_string(arg_name);
	local = 0;
	type = 0;
	}

EventHandler::~EventHandler()
	{
	Unref(local);
	delete [] name;
	}

FuncType* EventHandler::FType()
	{
	if ( type )
		return type;

	ID* id = lookup_ID(name, current_module.c_str());

	if ( ! id )
		return 0;

	if ( id->Type()->Tag() != TYPE_FUNC )
		return 0;

	return type = id->Type()->AsFuncType();
	}

void EventHandler::SetLocalHandler(Func* f)
	{
	if ( local )
		Unref(local);

	Ref(f);
	local = f;
	}

void EventHandler::Call(val_list* vl)
	{
	loop_over_list(receivers, i)
		remote_serializer->SendEvent(receivers[i]->ip,
						receivers[i]->port, name, vl);

	if ( local )
		local->Call(vl);
	else
		{
		loop_over_list(*vl, i)
			Unref((*vl)[i]);
		}
	}

void EventHandler::AddRemoteHandler(addr_type ip, uint16 port)
	{
	receivers.append(new Receiver(ip, port));
	}

void EventHandler::RemoveRemoteHandler(addr_type ip, uint16 port)
	{
	loop_over_list(receivers, i)
		{
		if ( receivers[i]->ip == ip && receivers[i]->port == port )
			delete receivers.remove_nth(i);
		}
	}

bool EventHandler::Serialize(Serializer *s) const
	{
	return s->Write(name);
	}

EventHandler* EventHandler::Unserialize(Serializer *s)
	{
	const char* name;
	if ( ! s->Read(&name, 0) )
		return false;

	EventHandler* h = event_registry->Lookup(name);
	if ( ! h )
		{
		h = new EventHandler(name);
		event_registry->Register(h);
		}

	return h;
	}
