# $Id: remote.bro,v 1.3 2003/10/10 00:27:46 vern Exp $
#
# Connect to remote Bros and request some of their events.

@load alert

type Peer: record {
	events: pattern;
	retry: interval;
};

const remote_peers_clear: table[addr, port] of Peer &redef;
const remote_peers_ssl: table[addr, port] of Peer &redef;

# redef remote_peers_ssl += {
# 	[127.0.0.1, 47756/tcp] = [$events = /.*/, $retry = 60 secs]
# };

global remote_log = open_log_file("remote");

function build_capture_filter_index(ip: addr, p: port) : string
	{
	return fmt("remote-%s-%d", ip, p );
	}

event remote_connection_established(ip: addr, p: port)
	{
	print remote_log, fmt("%.6f %s/%d Connection established.",
				current_time(), ip, p);

	if ( [ip, p] in remote_peers_clear )
		{
		request_remote_events(ip, p, remote_peers_clear[ip, p]$events);
		send_capture_filter(ip, p, default_pcap_filter);
		}

	if ( [ip, p] in remote_peers_ssl )
		{
		request_remote_events(ip, p, remote_peers_ssl[ip, p]$events);
		send_capture_filter(ip, p, default_pcap_filter);
		}
	}

event remote_connection_closed(ip: addr, p: port)
	{
	local i = build_capture_filter_index(ip, p);
	if ( i in capture_filters )
		{
		delete capture_filters[i];
		update_default_pcap_filter();
		}

	print remote_log, fmt("%.6f %s/%d Connection closed.",
				current_time(), ip, p);
	}

event remote_connection_error(ip: addr, p: port, reason: string)
	{
	print remote_log, fmt("%.6f %s/%d Error: %s",
				current_time(), ip, p, reason);
	}

event bro_init()
	{
	set_buf(remote_log, F);

	for ( [ip, p] in remote_peers_clear )
		connect_clear(ip, p, remote_peers_clear[ip, p]$retry);

	for ( [ip, p] in remote_peers_ssl )
		connect_ssl(ip, p, remote_peers_ssl[ip, p]$retry);
	}

event alert_action(a: alert_info, action: AlertAction)
	{
	# Forward remote alert to our local alarm system.
	if ( is_remote_event() )
		ALERT(a);
	}
